@@ -1,12 +1,13 @@
-import base64
+from Crypto import Random
 from Crypto.Cipher import AES
 from Crypto.Hash import SHA256
-from Crypto import Random
+
 
 def get_key(password):
     key = SHA256.new(password.encode()).digest()
     return key
 
+
 def encrypt(key, source):
     IV = Random.new().read(AES.block_size)  # generate IV
     encryptor = AES.new(key, AES.MODE_CBC, IV)
@@ -15,11 +16,14 @@ def encrypt(key, source):
     data = IV + encryptor.encrypt(source)  # store the IV at the beginning and encrypt
     return data
 
+
 def decrypt(key, source):
-    IV = source[:AES.block_size]  # extract the IV from the beginning
+    IV = source[: AES.block_size]  # extract the IV from the beginning
     decryptor = AES.new(key, AES.MODE_CBC, IV)
-    data = decryptor.decrypt(source[AES.block_size:])  # decrypt
+    data = decryptor.decrypt(source[AES.block_size :])  # decrypt
     padding = data[-1]  # pick the padding value from the end; Python 2.x: ord(data[-1])
-    if data[-padding:] != bytes([padding]) * padding:  # Python 2.x: chr(padding) * padding
+    if (
+        data[-padding:] != bytes([padding]) * padding
+    ):  # Python 2.x: chr(padding) * padding
         raise ValueError("Invalid padding...")
-    return data[:-padding]  # remove the padding
\ No newline at end of file
+    return data[:-padding]  # remove the padding
